@@ -487,12 +487,15 @@ typedef gint32	GTime;
 
 
 /* Portable endian checks and conversions
+ *
+ * glibconfig.h defines G_BYTE_ORDER which expands to one of
+ * the below macros.
  */
-
 #define G_LITTLE_ENDIAN 1234
 #define G_BIG_ENDIAN    4321
 #define G_PDP_ENDIAN    3412		/* unused, need specific PDP check */	
 
+
 /* Basic bit swapping functions
  */
 #define GUINT16_SWAP_LE_BE_CONSTANT(val)	((guint16) ( \
@@ -507,7 +510,6 @@ typedef gint32	GTime;
 /* Intel specific stuff for speed
  */
 #if defined (__i386__) && defined (__GNUC__) && __GNUC__ >= 2
-
 #  define GUINT16_SWAP_LE_BE_X86(val) \
      (__extension__					\
       ({ register guint16 __v;				\
@@ -518,9 +520,7 @@ typedef gint32	GTime;
 			      : "=r" (__v)		\
 			      : "0" ((guint16) (val)));	\
 	__v; }))
-
 #  define GUINT16_SWAP_LE_BE(val) (GUINT16_SWAP_LE_BE_X86 (val))
-
 #  if !defined(__i486__) && !defined(__i586__) \
       && !defined(__pentium__) && !defined(__i686__) && !defined(__pentiumpro__)
 #     define GUINT32_SWAP_LE_BE_X86(val) \
@@ -535,7 +535,6 @@ typedef gint32	GTime;
 			       : "=r" (__v)			\
 			       : "0" ((guint32) (val)));	\
 	__v; }))
-
 #  else /* 486 and higher has bswap */
 #     define GUINT32_SWAP_LE_BE_X86(val) \
         (__extension__						\
@@ -548,9 +547,7 @@ typedef gint32	GTime;
 			       : "0" ((guint32) (val)));	\
 	__v; }))
 #  endif /* processor specific 32-bit stuff */
-
 #  define GUINT32_SWAP_LE_BE(val) (GUINT32_SWAP_LE_BE_X86 (val))
-
 #else /* !__i386__ */
 #  define GUINT16_SWAP_LE_BE(val) (GUINT16_SWAP_LE_BE_CONSTANT (val))
 #  define GUINT32_SWAP_LE_BE(val) (GUINT32_SWAP_LE_BE_CONSTANT (val))
@@ -574,7 +571,6 @@ typedef gint32	GTime;
 	(guint64) G_GINT64_CONSTANT(0x00ff000000000000U)) >> 40) |	\
       (((guint64) (val) &						\
 	(guint64) G_GINT64_CONSTANT(0xff00000000000000U)) >> 56)))
-
 #  if defined (__i386__) && defined (__GNUC__) && __GNUC__ >= 2
 #    define GUINT64_SWAP_LE_BE_X86(val) \
 	(__extension__						\
@@ -591,9 +587,7 @@ typedef gint32	GTime;
 		__r.__l[1] = GUINT32_SWAP_LE_BE (__w.__l[0]);	\
 	      }							\
 	  __r.__ll; }))
-
 #    define GUINT64_SWAP_LE_BE(val) (GUINT64_SWAP_LE_BE_X86 (val))
-
 #  else /* !__i386__ */
 #    define GUINT64_SWAP_LE_BE(val) (GUINT64_SWAP_LE_BE_CONSTANT(val))
 #  endif
@@ -608,8 +602,8 @@ typedef gint32	GTime;
     (((guint32) (val) & (guint32) 0x00ff00ffU) << 8) | \
     (((guint32) (val) & (guint32) 0xff00ff00U) >> 8)))
 
-/* The TO_?E stuff is defined in glibconfig.h. The transformation is symmetric,
-   so the FROM just maps to the TO.
+/* The G*_TO_?E() macros are defined in glibconfig.h.
+ * The transformation is symmetric, so the FROM just maps to the TO.
  */
 #define GINT16_FROM_LE(val)	(GINT16_TO_LE (val))
 #define GUINT16_FROM_LE(val)	(GUINT16_TO_LE (val))
@@ -621,10 +615,10 @@ typedef gint32	GTime;
 #define GUINT32_FROM_BE(val)	(GUINT32_TO_BE (val))
 
 #ifdef G_HAVE_GINT64
-#define GINT64_FROM_LE(val)	(GINT32_TO_LE (val))
-#define GUINT64_FROM_LE(val)	(GUINT32_TO_LE (val))
-#define GINT64_FROM_BE(val)	(GINT32_TO_BE (val))
-#define GUINT64_FROM_BE(val)	(GUINT32_TO_BE (val))
+#define GINT64_FROM_LE(val)	(GINT64_TO_LE (val))
+#define GUINT64_FROM_LE(val)	(GUINT64_TO_LE (val))
+#define GINT64_FROM_BE(val)	(GINT64_TO_BE (val))
+#define GUINT64_FROM_BE(val)	(GUINT64_TO_BE (val))
 #endif
 
 #define GLONG_FROM_LE(val)	(GLONG_TO_LE (val))
@@ -637,6 +631,7 @@ typedef gint32	GTime;
 #define GINT_FROM_BE(val)	(GINT_TO_BE (val))
 #define GUINT_FROM_BE(val)	(GUINT_TO_BE (val))
 
+
 /* Portable versions of host-network order stuff
  */
 #define g_ntohl(val) (GUINT32_FROM_BE (val))
@@ -665,9 +660,9 @@ GUTILS_C_VAR const guint glib_micro_version;
 GUTILS_C_VAR const guint glib_interface_age;
 GUTILS_C_VAR const guint glib_binary_age;
 
+
 /* Forward declarations of glib types.
  */
-
 typedef struct _GAllocator	GAllocator;
 typedef struct _GArray		GArray;
 typedef struct _GByteArray	GByteArray;
@@ -694,7 +689,6 @@ typedef struct _GTuples		GTuples;
 typedef union  _GTokenValue	GTokenValue;
 typedef struct _GIOChannel	GIOChannel;
 
-
 typedef enum
 {
   G_TRAVERSE_LEAFS	= 1 << 0,
@@ -1189,9 +1183,14 @@ GHook*	 g_hook_find_func_data		(GHookList		*hook_list,
 					 gboolean		 need_valids,
 					 gpointer		 func,
 					 gpointer		 data);
+/* return the first valid hook, and increment its reference count */
 GHook*	 g_hook_first_valid		(GHookList		*hook_list,
 					 gboolean		 may_be_in_call);
-GHook*	 g_hook_next_valid		(GHook			*hook,
+/* return the next valid hook with incremented reference count, and
+ * decrement the reference count of the original hook
+ */
+GHook*	 g_hook_next_valid		(GHookList		*hook_list,
+					 GHook			*hook,
 					 gboolean		 may_be_in_call);
 
 /* GHookCompareFunc implementation to insert hooks sorted by their id */
@@ -2300,31 +2299,12 @@ typedef enum
 } GSeekType;
 typedef enum
 {
-  G_IO_IN  
-#ifdef POLLIN
-            = POLLIN
-#endif
-  , G_IO_OUT  
-#ifdef POLLOUT
-            = POLLOUT
-#endif
-  , G_IO_PRI  
-#ifdef POLLPRI
-            = POLLPRI
-#endif
-
-  , G_IO_ERR  
-#ifdef POLLERR
-           = POLLERR
-#endif
-  , G_IO_HUP  
-#ifdef POLLHUP
-           = POLLHUP
-#endif
-  , G_IO_NVAL  
-#ifdef POLLNVAL
-           = POLLNVAL
-#endif
+  G_IO_IN	GLIB_SYSDEF_POLLIN,
+  G_IO_OUT	GLIB_SYSDEF_POLLOUT,
+  G_IO_PRI	GLIB_SYSDEF_POLLPRI,
+  G_IO_ERR	GLIB_SYSDEF_POLLERR,
+  G_IO_HUP	GLIB_SYSDEF_POLLHUP,
+  G_IO_NVAL	GLIB_SYSDEF_POLLNVAL
 } GIOCondition;
 
 struct _GIOChannel
@@ -2411,6 +2391,14 @@ struct _GSourceFuncs
   GDestroyNotify destroy;
 };
 
+/* Standard priorities */
+
+#define G_PRIORITY_HIGH            -100
+#define G_PRIORITY_DEFAULT          0
+#define G_PRIORITY_HIGH_IDLE        100
+#define G_PRIORITY_DEFAULT_IDLE     200
+#define G_PRIORITY_LOW	            300
+
 typedef gboolean (*GSourceFunc) (gpointer data);
 
 /* Hooks for adding to the main loop */
@@ -2427,10 +2415,11 @@ void g_source_remove_by_source_data (gpointer       source_data);
 void g_get_current_time		    (GTimeVal	   *result);
 
 /* Running the main loop */
-GMainLoop*	g_main_new		(void);
+GMainLoop*	g_main_new		(gboolean	 is_running);
 void		g_main_run		(GMainLoop	*loop);
 void		g_main_quit		(GMainLoop	*loop);
 void		g_main_destroy		(GMainLoop	*loop);
+gboolean	g_main_is_running	(GMainLoop	*loop);
 
 /* Run a single iteration of the mainloop. If block is FALSE,
  * will never block
@@ -2472,9 +2461,9 @@ struct _GPollFD
   gushort 	revents;
 };
 
-void        g_main_poll_add          (gint        priority,
-				      GPollFD    *fd);
-void        g_main_poll_remove       (GPollFD    *fd);
+void        g_main_add_poll          (GPollFD    *fd,
+				      gint        priority);
+void        g_main_remove_poll       (GPollFD    *fd);
 void        g_main_set_poll_func     (GPollFunc   func);
 
 GIOChannel* g_io_channel_unix_new    (int         fd);
@@ -2678,7 +2667,7 @@ void     g_static_private_set (GStaticPrivate	*private_key,
  * such mutexes can be done with G_LOCK(), G_UNLOCK() and G_TRYLOCK()
  * respectively.
  */
-extern gboolean glib_dummy_decl;
+extern void glib_dummy_decl (void);
 #define G_LOCK_NAME(name)		(g__ ## name ## _lock)
 #ifdef	G_THREADS_ENABLED
 #  define G_LOCK_DECLARE_STATIC(name)	static G_LOCK_DECLARE (name)
@@ -2712,8 +2701,8 @@ extern gboolean glib_dummy_decl;
 #    define G_TRYLOCK(name) g_static_mutex_trylock (G_LOCK_NAME (name))
 #  endif /* !G_DEBUG_LOCKS */
 #else	/* !G_THREADS_ENABLED */
-#  define G_LOCK_DECLARE_STATIC(name)	extern gboolean glib_dummy_decl
-#  define G_LOCK_DECLARE(name)		extern gboolean glib_dummy_decl
+#  define G_LOCK_DECLARE_STATIC(name)	extern void glib_dummy_decl (void)
+#  define G_LOCK_DECLARE(name)		extern void glib_dummy_decl (void)
 #  define G_LOCK(name)
 #  define G_UNLOCK(name)
 #  define G_TRYLOCK(name)		(FALSE)
