@@ -25,7 +25,11 @@
  */
 
 #include "glib.h"
+#include <sys/types.h>
 #include <sys/time.h>
+#ifdef	GLIB_HAVE_SYS_POLL_H
+#include <sys/poll.h>
+#endif	/* GLIB_HAVE_SYS_POLL_H */
 #include <unistd.h>
 #include <errno.h>
 #include "config.h"
@@ -52,7 +56,7 @@ struct _GSource
 
 struct _GMainLoop
 {
-  gboolean flag;
+  gboolean is_running;
 };
 
 struct _GIdleData
@@ -79,7 +83,7 @@ struct _GPollRec
 static void     g_main_poll               (gint      timeout,
 					   gboolean  use_priority, 
 					   gint      priority);
-static void     g_main_add_unlocking_poll (gint      priority,
+static void     g_main_add_poll_unlocked  (gint      priority,
 					   GPollFD  *fd);
 
 static gboolean g_timeout_prepare      (gpointer  source_data, 
@@ -283,7 +287,7 @@ g_source_remove (guint tag)
   if (hook)
     {
       GSource *source = (GSource *)hook;
-      
+
       ((GSourceFuncs *) source->hook.func)->destroy (source->source_data);
       g_hook_destroy_link (&source_list, hook);
     }
@@ -342,9 +346,14 @@ g_source_remove_by_source_data (gpointer source_data)
 void
 g_get_current_time (GTimeVal *result)
 {
+  struct timeval r;
   g_return_if_fail (result != NULL);
 
-  gettimeofday ((struct timeval *) result, NULL);
+  /*this is required on alpha, there the timeval structs are int's
+    not longs and a cast only would fail horribly*/
+  gettimeofday (&r, NULL);
+  result->tv_sec = r.tv_sec;
+  result->tv_usec = r.tv_usec;
 }
 
 /* Running the main loop */
@@ -365,6 +374,7 @@ g_main_dispatch (GTimeVal *current_time)
 
       if (G_HOOK_IS_VALID (source))
 	{
+	  gboolean was_in_call;
 	  gpointer hook_data = source->hook.data;
 	  gpointer source_data = source->source_data;
 	  gboolean (*dispatch) (gpointer,
@@ -373,6 +383,7 @@ g_main_dispatch (GTimeVal *current_time)
 
 	  dispatch = ((GSourceFuncs *) source->hook.func)->dispatch;
 	  
+	  was_in_call = G_HOOK_IN_CALL (source);
 	  source->hook.flags |= G_HOOK_FLAG_IN_CALL;
 
 	  G_UNLOCK (main_loop);
@@ -381,27 +392,57 @@ g_main_dispatch (GTimeVal *current_time)
 				     hook_data);
 	  G_LOCK (main_loop);
 
-	  source->hook.flags &= ~G_HOOK_FLAG_IN_CALL;
+	  if (!was_in_call)
+	    source->hook.flags &= ~G_HOOK_FLAG_IN_CALL;
 	  
-	  if (need_destroy)
-	    g_hook_destroy_link (&source_list, (GHook *) source);
+	  if (need_destroy && G_HOOK_IS_VALID (source))
+	    {
+	      ((GSourceFuncs *) source->hook.func)->destroy (source->source_data);
+	      g_hook_destroy_link (&source_list, (GHook *) source);
+	    }
 	}
 
       g_hook_unref (&source_list, (GHook *)source);
     }
 }
 
-/* Run a single iteration of the mainloop, or, if !dispatch
- * check to see if any events need dispatching, but don't
- * run the loop.
+/* g_main_iterate () runs a single iteration of the mainloop, or,
+ * if !dispatch checks to see if any sources need dispatching.
+ * basic algorithm for dispatch=TRUE:
+ *
+ * 1) while the list of currently pending sources is non-empty,
+ *    we call (*dispatch) on those that are !IN_CALL or can_recurse,
+ *    removing sources from the list after each returns.
+ *    the return value of (*dispatch) determines whether the source
+ *    itself is kept alive.
+ *
+ * 2) call (*prepare) for sources that are not yet SOURCE_READY and
+ *    are !IN_CALL or can_recurse. a return value of TRUE determines
+ *    that the source would like to be dispatched immediatedly, it
+ *    is then flagged as SOURCE_READY.
+ *
+ * 3) poll with the pollfds from all sources at the priority of the
+ *    first source flagged as SOURCE_READY. if there are any sources
+ *    flagged as SOURCE_READY, we use a timeout of 0 or the minimum
+ *    of all timouts otherwise.
+ *
+ * 4) for each source !IN_CALL or can_recurse, if SOURCE_READY or
+ *    (*check) returns true, add the source to the pending list.
+ *    once one source returns true, stop after checking all sources
+ *    at that priority.
+ *
+ * 5) while the list of currently pending sources is non-empty,
+ *    call (*dispatch) on each source, removing the source
+ *    after the call.
+ *
  */
 static gboolean
 g_main_iterate (gboolean block,
 		gboolean dispatch)
 {
   GHook *hook;
-  GTimeVal current_time;
-  gint nready = 0;
+  GTimeVal current_time  ={ 0, 0 };
+  gint n_ready = 0;
   gint current_priority = 0;
   gint timeout;
   gboolean retval = FALSE;
@@ -419,6 +460,7 @@ g_main_iterate (gboolean block,
 	g_main_dispatch (&current_time);
       
       G_UNLOCK (main_loop);
+
       return TRUE;
     }
 
@@ -431,32 +473,35 @@ g_main_iterate (gboolean block,
     {
       GSource *source = (GSource *)hook;
       GHook *tmp;
-      gint source_timeout;
+      gint source_timeout = -1;
 
-      if ((nready > 0) && (source->priority > current_priority))
+      if ((n_ready > 0) && (source->priority > current_priority))
 	break;
-      if (!(hook->flags & G_SOURCE_CAN_RECURSE) && G_HOOK_IN_CALL (hook))
+      if (G_HOOK_IN_CALL (hook) && !(hook->flags & G_SOURCE_CAN_RECURSE))
 	{
-	  hook = g_hook_next_valid (hook, TRUE);
+	  hook = g_hook_next_valid (&source_list, hook, TRUE);
 	  continue;
 	}
 
       g_hook_ref (&source_list, hook);
-
-      if (((GSourceFuncs *)hook->func)->prepare (source->source_data,
-						 &current_time,
-						 &source_timeout))
+      
+      if (hook->flags & G_SOURCE_READY ||
+	  ((GSourceFuncs *) hook->func)->prepare (source->source_data,
+						  &current_time,
+						  &source_timeout))
 	{
 	  if (!dispatch)
 	    {
+	      hook->flags |= G_SOURCE_READY;
 	      g_hook_unref (&source_list, hook);
 	      G_UNLOCK (main_loop);
+
 	      return TRUE;
 	    }
 	  else
 	    {
 	      hook->flags |= G_SOURCE_READY;
-	      nready++;
+	      n_ready++;
 	      current_priority = source->priority;
 	      timeout = 0;
 	    }
@@ -470,7 +515,7 @@ g_main_iterate (gboolean block,
 	    timeout = MIN (timeout, source_timeout);
 	}
 
-      tmp = g_hook_next_valid (hook, TRUE);
+      tmp = g_hook_next_valid (&source_list, hook, TRUE);
       
       g_hook_unref (&source_list, hook);
       hook = tmp;
@@ -478,11 +523,11 @@ g_main_iterate (gboolean block,
 
   /* poll(), if necessary */
 
-  g_main_poll (timeout, nready > 0, current_priority);
+  g_main_poll (timeout, n_ready > 0, current_priority);
 
   /* Check to see what sources need to be dispatched */
 
-  nready = 0;
+  n_ready = 0;
   
   hook = g_hook_first_valid (&source_list, TRUE);
   while (hook)
@@ -490,19 +535,19 @@ g_main_iterate (gboolean block,
       GSource *source = (GSource *)hook;
       GHook *tmp;
 
-      if ((nready > 0) && (source->priority > current_priority))
+      if ((n_ready > 0) && (source->priority > current_priority))
 	break;
-      if (!(hook->flags & G_SOURCE_CAN_RECURSE) && G_HOOK_IN_CALL (hook))
+      if (G_HOOK_IN_CALL (hook) && !(hook->flags & G_SOURCE_CAN_RECURSE))
 	{
-	  hook = g_hook_next_valid (hook, TRUE);
+	  hook = g_hook_next_valid (&source_list, hook, TRUE);
 	  continue;
 	}
 
       g_hook_ref (&source_list, hook);
 
-      if ((hook->flags & G_SOURCE_READY) ||
-	  ((GSourceFuncs *)hook->func)->check (source->source_data,
-					       &current_time))
+      if (hook->flags & G_SOURCE_READY ||
+	  ((GSourceFuncs *) hook->func)->check (source->source_data,
+						&current_time))
 	{
 	  if (dispatch)
 	    {
@@ -510,17 +555,18 @@ g_main_iterate (gboolean block,
 	      g_hook_ref (&source_list, hook);
 	      pending_dispatches = g_slist_prepend (pending_dispatches, source);
 	      current_priority = source->priority;
-	      nready++;
+	      n_ready++;
 	    }
 	  else
 	    {
 	      g_hook_unref (&source_list, hook);
 	      G_UNLOCK (main_loop);
+
 	      return TRUE;
 	    }
 	}
       
-      tmp = g_hook_next_valid (hook, TRUE);
+      tmp = g_hook_next_valid (&source_list, hook, TRUE);
       
       g_hook_unref (&source_list, hook);
       hook = tmp;
@@ -557,40 +603,58 @@ g_main_iteration (gboolean block)
   return g_main_iterate (block, TRUE);
 }
 
-GMainLoop *
-g_main_new ()
+GMainLoop*
+g_main_new (gboolean is_running)
 {
-  GMainLoop *result = g_new (GMainLoop, 1);
-  result->flag = FALSE;
+  GMainLoop *loop;
 
-  return result;
+  loop = g_new0 (GMainLoop, 1);
+  loop->is_running = is_running != FALSE;
+
+  return loop;
 }
 
 void 
 g_main_run (GMainLoop *loop)
 {
-  loop->flag = FALSE;
-  while (!loop->flag)
+  g_return_if_fail (loop != NULL);
+
+  loop->is_running = TRUE;
+  while (loop->is_running)
     g_main_iterate (TRUE, TRUE);
 }
 
 void 
 g_main_quit (GMainLoop *loop)
 {
-  loop->flag = TRUE;
+  g_return_if_fail (loop != NULL);
+
+  loop->is_running = FALSE;
 }
 
 void 
 g_main_destroy (GMainLoop *loop)
 {
+  g_return_if_fail (loop != NULL);
+
   g_free (loop);
 }
 
+gboolean
+g_main_is_running (GMainLoop *loop)
+{
+  g_return_val_if_fail (loop != NULL, FALSE);
+
+  return loop->is_running;
+}
+
 /* HOLDS: main_loop_lock */
 static void
-g_main_poll (gint timeout, gboolean use_priority, gint priority)
+g_main_poll (gint     timeout,
+	     gboolean use_priority,
+	     gint     priority)
 {
-  GPollFD *fd_array = g_new (GPollFD, n_poll_records);
+  GPollFD *fd_array;
   GPollRec *pollrec;
 
   gint i;
@@ -600,13 +664,15 @@ g_main_poll (gint timeout, gboolean use_priority, gint priority)
     {
       if (pipe (wake_up_pipe) < 0)
 	g_error ("Cannot create pipe main loop wake-up: %s\n",
-		 g_strerror(errno));
+		 g_strerror (errno));
 
       wake_up_rec.fd = wake_up_pipe[0];
       wake_up_rec.events = G_IO_IN;
-      g_main_add_unlocking_poll (0, &wake_up_rec);
+      g_main_add_poll_unlocked (0, &wake_up_rec);
     }
   
+  fd_array = g_new (GPollFD, n_poll_records);
+ 
   pollrec = poll_records;
   i = 0;
   while (pollrec && (!use_priority || priority >= pollrec->priority))
@@ -647,18 +713,18 @@ g_main_poll (gint timeout, gboolean use_priority, gint priority)
 }
 
 void 
-g_main_add_poll (gint     priority,
-		 GPollFD *fd)
+g_main_add_poll (GPollFD *fd,
+		 gint     priority)
 {
   G_LOCK (main_loop);
-  g_main_add_unlocking_poll (priority, fd);
+  g_main_add_poll_unlocked (priority, fd);
   G_UNLOCK (main_loop);
 }
 
 /* HOLDS: main_loop_lock */
 static void 
-g_main_add_unlocking_poll (gint     priority,
-			   GPollFD *fd)
+g_main_add_poll_unlocked (gint     priority,
+			  GPollFD *fd)
 {
   GPollRec *lastrec, *pollrec, *newrec;
 
@@ -777,8 +843,11 @@ g_timeout_dispatch (gpointer source_data,
 
   if (data->callback(user_data))
     {
-      data->expiration.tv_sec = current_time->tv_sec;
-      data->expiration.tv_usec = current_time->tv_usec + data->interval * 1000;
+      guint seconds = data->interval / 1000;
+      guint msecs = data->interval - seconds * 1000;
+
+      data->expiration.tv_sec = current_time->tv_sec + seconds;
+      data->expiration.tv_usec = current_time->tv_usec + msecs * 1000;
       if (data->expiration.tv_usec >= 1000000)
 	{
 	  data->expiration.tv_usec -= 1000000;
@@ -797,13 +866,19 @@ g_timeout_add_full (gint           priority,
 		    gpointer       data,
 		    GDestroyNotify notify)
 {
+  guint seconds;
+  guint msecs;
   GTimeoutData *timeout_data = g_new (GTimeoutData, 1);
 
   timeout_data->interval = interval;
   timeout_data->callback = function;
   g_get_current_time (&timeout_data->expiration);
 
-  timeout_data->expiration.tv_usec += timeout_data->interval * 1000;
+  seconds = timeout_data->interval / 1000;
+  msecs = timeout_data->interval - seconds * 1000;
+
+  timeout_data->expiration.tv_sec += seconds;
+  timeout_data->expiration.tv_usec += msecs * 1000;
   if (timeout_data->expiration.tv_usec >= 1000000)
     {
       timeout_data->expiration.tv_usec -= 1000000;
@@ -818,7 +893,8 @@ g_timeout_add (guint32        interval,
 	       GSourceFunc    function,
 	       gpointer       data)
 {
-  return g_timeout_add_full (0, interval, function, data, NULL);
+  return g_timeout_add_full (G_PRIORITY_DEFAULT, 
+			     interval, function, data, NULL);
 }
 
 /* Idle functions */
@@ -866,5 +942,5 @@ guint
 g_idle_add (GSourceFunc    function,
 	    gpointer       data)
 {
-  return g_idle_add_full (0, function, data, NULL);
+  return g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, function, data, NULL);
 }
